// Copyright 2022 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package configuration

import (
	"os"
	"reflect"
	"strconv"
	"strings"
)

func (s *Settings[C]) InitFromEnv(prefix string) *Settings[C] {

	s.Lock()
	defer s.Unlock()

	defer func() {
		s.notifyErrorHooks()
	}()

	if prefix != "" {
		prefix = strings.ToUpper(prefix)
		if !strings.HasSuffix(prefix, "_") {
			prefix += "_"
		}
	}

	err := runOnTags(&s.config, []string{envTagKey}, func(k string, field reflect.Value) {

		if !field.CanSet() {
			return
		}

		k = prefix + strings.ToUpper(k)

		v, found := os.LookupEnv(k)
		if !found {
			return // env not found
		}

		t := reflect.TypeOf(v)

		switch field.Kind() {
		case reflect.String:
			field.SetString(v)
		case reflect.Int:
			intVar, err := strconv.Atoi(v)
			if err != nil {
				s.errors = append(s.errors, err)
				return
			}
			field.SetInt(int64(intVar))
		case reflect.Bool:
			field.SetBool(v == "true")
		default:
			s.errors = append(s.errors, newMismatchedTypeError(t, field.Type()))
		}
	})

	if err != nil {
		s.errors = append(s.errors, err)
	}

	return s

}
